/*

FastPACS

2006-2013 Precision Diagnostic Systems, Inc.
Maple Park, IL 60151 USA

*/
//==============================
// QueryStringValue Class
//==============================
const UInt32 kImageSpacing = 4;

//==============================
// QueryStringValue Class
//==============================
class QueryStringValue : QueryStringValueBase {

	public readonly String volume;
	public readonly String path;
	public readonly String series_uid;
	public readonly UInt32 window_width;
	public readonly UInt32 window_height;
	public readonly String fit;
	public readonly String layout;
	public readonly String order;
	public readonly String voi;
	public readonly Single zoom;
	public readonly Int32 zx;
	public readonly Int32 zy;
	public readonly String action;
	public readonly String overlay;
	public readonly Int32 first_index;

	public readonly bool bad;

	public QueryStringValue(HttpRequest req, BrowserCheck browser, StringDictionary pref)
		: base(req) {

		UInt32 width_margin = 2;
		UInt32 height_margin = 2;

		if (browser.type == "NN") {
			if (browser.os == "M") {
				width_margin = 8;
				height_margin = 8;
			} else if (browser.os == "W") {
				width_margin = 4;
				height_margin = 4;
			}
		} else if (browser.type == "IE") {
			if (browser.os == "M") {
				width_margin = 16;
				height_margin = 16;
			}
		}

		this.volume = get_opt_str("VOLUME", null);
		this.path = get_opt_str("PATH", null);
		this.series_uid = get_opt_str("SERIES", null);
		this.window_width = (UInt32)(get_opt_int("WW", 600) - width_margin);
		this.window_height = (UInt32)(get_opt_int("WH", 400) - height_margin);

		this.voi = get_opt_str("VOI", null);
		this.zoom = Single.Parse(get_opt_str("ZOOM", "1.0"));
		this.zx = get_opt_int("ZX", -1);
		this.zy = get_opt_int("ZY", -1);
		this.first_index = get_opt_int("FI", -1);

		this.layout = get_val("LAYOUT", pref, default_layout, null);
		this.fit = get_val("FIT", pref, default_fit, "2");
		this.order = get_val("ORDER", pref, default_order, "N,A");
		this.overlay = get_val("OVERLAY", pref, default_overlay, "D");
		this.action = get_val("ACTION", pref, default_action, "P");

		this.bad = this.volume == null || this.path == null || this.series_uid == null;
	}

}

//==============================
//
//
//
//==============================
class LayoutEntry {
	public readonly UInt32 rows, cols;
	public readonly UInt32 pages;
	public readonly UInt32 width, height;
	public readonly UInt32 num_images;
	public readonly Double scale;

	//==============================
	//
	//==============================
	public LayoutEntry(UInt32 rows, UInt32 cols, Double scale, UInt32 pages, UInt32 width, UInt32 height) {
		this.rows = rows;
		this.cols = cols;
		this.scale = scale;
		this.pages = pages;
		this.width = width;
		this.height = height;
		this.num_images = rows * cols;
	}
}

//==============================
//
//
//
//==============================
class LayoutTableObject {

	//==============================
	//
	//
	//
	//==============================
	abstract class LayoutGenerator {
		protected UInt32 canvasWidth, canvasHeight, imageHeight, imageWidth, numImages;

		//==============================
		//
		//==============================
		public LayoutGenerator(UInt32 cw, UInt32 ch, UInt32 ih, UInt32 iw, UInt32 ni) {
			canvasWidth = cw;
			canvasHeight = ch;
			imageHeight = ih;
			imageWidth = iw;
			numImages = ni;
		}

		//==============================
		//
		//==============================
		public abstract LayoutEntry Gen(UInt32 rows, UInt32 cols);
	}

	//==============================
	//
	//
	//
	//==============================
	class FitLayout : LayoutGenerator {

		//==============================
		//
		//==============================
		public FitLayout(UInt32 cw, UInt32 ch, UInt32 ih, UInt32 iw, UInt32 ni)
			: base(cw, ch, ih, iw, ni) {}

		//==============================
		//
		//==============================
		public override LayoutEntry Gen(UInt32 rows, UInt32 cols) {
			UInt32 fit_factor = 8;
			UInt32 pages = (UInt32)Math.Ceiling((Double)numImages / (Double)(rows * cols));
			UInt32 width =  (UInt32)Math.Floor((Double)(canvasWidth - kImageSpacing * (cols - 1)) / (Double)cols / (Double)fit_factor) * fit_factor;
			UInt32 height = (UInt32)Math.Floor((Double)(canvasHeight - kImageSpacing * (rows - 1)) / (Double)rows / (Double)fit_factor) * fit_factor;
			Double xscale = (Double)width / (Double)imageWidth;
			Double yscale = (Double)height / (Double)imageHeight;
			Double scale = Math.Min(xscale, yscale);
			width = (UInt32)Math.Ceiling(imageWidth * scale);
			height = (UInt32)Math.Ceiling(imageHeight * scale);

			return new LayoutEntry(rows, cols, scale, pages, width, height);
		}
	}

	//==============================
	//
	//
	//
	//==============================
	class TwoLayout : LayoutGenerator {

		//==============================
		//
		//==============================
		public TwoLayout(UInt32 cw, UInt32 ch, UInt32 ih, UInt32 iw, UInt32 ni)
			: base(cw, ch, ih, iw, ni) {}

		//==============================
		//
		//==============================
		Double calc_scale2(Double in_scale) {
			Double out_scale = 1.0;
			if (in_scale < out_scale)
				while (in_scale < out_scale)
					out_scale /= 2;
			else if (in_scale > out_scale)
				while (in_scale > out_scale * 2)
					out_scale *= 2;
			return out_scale;
		}

		//==============================
		//
		//==============================
		public override LayoutEntry Gen(UInt32 rows, UInt32 cols) {
			UInt32 pages = (UInt32)Math.Ceiling(Convert.ToDouble(numImages / (rows * cols)));
			Double yscale = calc_scale2((Double)(canvasHeight - kImageSpacing * (rows - 1)) / (Double)(imageHeight * rows));
			Double xscale = calc_scale2((Double)(canvasWidth - kImageSpacing * (cols - 1)) / (Double)(imageWidth * cols));
			Double scale = Math.Min(xscale, yscale);
			UInt32 width = (UInt32)Math.Ceiling((Double)imageWidth * scale);
			UInt32 height = (UInt32)Math.Ceiling((Double)imageHeight * scale);

			return new LayoutEntry(rows, cols, scale, pages, width, height);
		}
	}

	public readonly ArrayList table;
	public readonly LayoutEntry layout;
	public readonly Boolean noscroll;

	//==============================
	//
	//==============================
	public LayoutTableObject(DicomStudy dom, QueryStringValue qsv, UInt32 num_images) {

		LayoutGenerator lg;
		if (qsv.fit == "F")
			lg = new FitLayout(qsv.window_width - kStripWidth, qsv.window_height - kInfoBarHeight - kMenuBarHeight, get_data_US(null, dom, 0x00280010, 0), get_data_US(null, dom, 0x00280011, 0), num_images);
		else
			lg = new TwoLayout(qsv.window_width - kStripWidth, qsv.window_height - kInfoBarHeight - kMenuBarHeight, get_data_US(null, dom, 0x00280010, 0), get_data_US(null, dom, 0x00280011, 0), num_images);

		table = new ArrayList();

		table.Add(lg.Gen(1, 1));

		table.Add(lg.Gen(2, 1));
		table.Add(lg.Gen(1, 2));

		table.Add(lg.Gen(2, 2));

		table.Add(lg.Gen(3, 2));
		table.Add(lg.Gen(2, 3));

		table.Add(lg.Gen(3, 3));

		table.Add(lg.Gen(4, 3));
		table.Add(lg.Gen(3, 4));

		table.Add(lg.Gen(4, 4));

		table.Add(lg.Gen(5, 4));
		table.Add(lg.Gen(4, 5));

		this.layout = null;
		if (qsv.layout != null) {
			String[] rc = qsv.layout.Split(new Char[] {','});
			UInt32 r = UInt32.Parse(rc[0]);
			UInt32 c = UInt32.Parse(rc[1]);
			if (r * c > num_images)
				r = (UInt32)((num_images / c) + (num_images % c == 0 ? 0 : 1));
			foreach (LayoutEntry lo in table) {
				if (lo.rows == r && lo.cols == c) {
					this.layout = lo;
					break;
				}
			}
		}
		if (this.layout == null) {
			ArrayList scoreArray = new ArrayList();
			foreach (LayoutEntry lo in table) {
				UInt32 laycount = lo.rows * lo.cols;
				if (laycount > num_images)
					break;
				if (laycount > 12)
					break;
				scoreArray.Add(lo.width * lo.height * laycount);
			}
			UInt32 max = 0;
			int index = 0;
			for (int i = 0; i < scoreArray.Count; i++) {
				UInt32 score = (UInt32)scoreArray[i];
				if (max == 0) {
					max = score;
				} else if (score >= max) {
					max = score;
					index = i;
				}
			}
			this.layout = (LayoutEntry)table[index];
		}

		this.noscroll = num_images <= (this.layout.rows * this.layout.cols);
	}

	//==============================
	//
	//==============================
	public LayoutEntry this[int index] {
		get { return (LayoutEntry)table[index]; }
	}

}

//==============================
//
//==============================
class SliderPanelObject {

	public readonly Double icon_step;
	public readonly UInt32 num_icons;
	public readonly UInt32 panel_height;

	public readonly UInt32 ind_height;
	public readonly Double ind_step;

	public readonly UInt32 num_image_box;

	public SliderPanelObject(QueryStringValue qsv, UInt32 numImages, LayoutTableObject layoutTable) {

		UInt32 height = qsv.window_height - kInfoBarHeight - kMenuBarHeight;
		UInt32 width = kStripWidth;
		UInt32 numImageBox = layoutTable.layout.num_images;

		UInt32 panel_height = height/* / 2*/ - width*2;

		UInt32 numIcons = numImages;
		if (numIcons * width > panel_height)
			numIcons = (UInt32)Math.Floor((Double)panel_height / (Double)width);

		Double step = 1.0;
		if (numIcons != numImages && numIcons > 1)
			step = (numImages - 1) / (numIcons - 1);

		this.icon_step = numImages / numIcons;
		this.num_icons = numIcons;
		this.num_image_box = numImageBox;
		this.panel_height = numIcons * width;

		if (layoutTable.noscroll) {
			this.ind_height = this.panel_height;
			this.ind_step = 0;
		} else {
			this.ind_height = (UInt32)Math.Floor((Double)this.panel_height * ((Double)numImageBox / (Double)numImages));
			this.ind_step = (Double)(this.panel_height - this.ind_height) / (Double)(numImages - numImageBox);
		}
	}
}



//==============================
//
//==============================
DicomStudy loadDOM(QueryStringValue qsv, XmlDocument config_dom) {
	String volume = qsv.volume;
	String path = qsv.path;
	String series_uid = qsv.series_uid;

	DicomStudy dom = null;
	StudyCache dw_obj = (StudyCache)Application.Get("StudyDOM");
	XmlNode vol = config_dom.SelectSingleNode("/DIOWaveConfiguration/VOLUME[@label=\"" + volume + "\"]");
	String dir = vol.SelectSingleNode("directory").InnerText;
	String studyLoc = vol.SelectSingleNode("location").InnerText + "/" + path;
	if (dir.StartsWith("\\")) {
		dom = dw_obj.GetRemoteDicomStudy("http://localhost/DIOWave/scripts/dio-study.dll" + studyLoc);
	} else {
		dom = dw_obj.GetDicomStudy(studyLoc);
	}

	DataElementObject series_uid_obj = null;
	foreach (DataElementObject deo in dom.el) {
		if (deo.tag != 0x0020000E)
			continue;
		if ((String)deo.value != series_uid)
			continue;
		series_uid_obj = deo;
		break;
	}
	if (series_uid_obj == null)
		return null;

	ArrayList new_fl = new ArrayList();
	foreach (SOPInstanceObject f in dom.fl) {
		bool found = false;
		foreach (DataElementObject deo in f.data) {
			if (deo.tag < 0x0020000E)
				continue;
			else if (deo.tag > 0x0020000E)
				break;
			if (deo != series_uid_obj)
				continue;
			found = true;
			break;
		}
		if (found) {
			new_fl.Add(f);
		}
	}
	dom.fl = new_fl;

	return dom;
}

//==============================
//
//==============================
void print_study_info_ex(DicomStudy dom, UInt32 num_images) {
	Writer.Write("#" + get_data_IS(null, dom, 0x00200011, String.Empty) + " [" + num_images + "]");
}

//==============================
//
//==============================
void printImageList(ArrayList image_list, ImageSpaceObject s) {

	Writer.Write("var IL = new Array(" + image_list.Count + ");\n");
	UInt32 i = 0;
	foreach (ImageObject img in image_list)
		Writer.Write("IL[" + i++ + "] = new IR(" + img.new_args(s) + ");\n");
}

//==============================
//
//==============================
void printStrip(UInt32 height, UInt32 width, UInt32 numImages, SliderPanelObject sliderPanel, QueryStringValue qsv) {
	if (numImages == 1) {
		Writer.Write("<TABLE ALIGN='CENTER' HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' WIDTH='" + width + "' HEIGHT='" + height + "'><TR><TD>");
		Writer.Write("<IMG SRC='/DIOWave/grayscale.gif' HEIGHT='" + height + "' WIDTH='" + width + "' BORDER='0' HSPACE='0' VSPACE='0'>");
		Writer.Write("</TD></TR></TABLE>\n");
		return;
	}

	Int32 firstIndex=qsv.first_index>=0?qsv.first_index:0;
	Double sliderHeight = width;
	Writer.Write("<DIV Style='width:"+width+"px;height:"
				 +sliderHeight*sliderPanel.num_icons+"px;overflow:hidden;position:absolute;margin-top:"
				 +(width+8)+"px'>");
	double tmpHeight = sliderHeight*sliderPanel.num_image_box;
	Writer.Write("<DIV id='SLIDER-BOX' style='width:"+(width-2)
				 +"px;height:"+tmpHeight
				 +"px;position:absolute;margin-top:"+firstIndex*sliderHeight
				 +"px'>");
	if (m_browser.version >= 5.5)
		Writer.Write("<div style='width:100%;height:100%;opacity:0.0;filter:alpha(opacity=0);background:#000000'></div>");
	Writer.Write("</DIV>");
	Writer.Write("</DIV>");

	Writer.Write("<TABLE ALIGN='CENTER' HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' WIDTH='" + width + "' HEIGHT='" + height + "'>\n");

	Writer.Write("<TR><TD CLASS='ARROW' HEIGHT='" + (width / 2).ToString() + "'><A HREF='javascript:Jump(-1);'><IMG BORDER='0' WIDTH='" + width + "' HEIGHT='" + (width / 2).ToString() + "' SRC='/DIOWave/uparrow.gif'></A></TD></TR>\n");
	Writer.Write("<TR><TD CLASS='ARROW' HEIGHT='" + (width / 2).ToString() + "'><A HREF='javascript:Jump(1);'><IMG BORDER='0' WIDTH='" + width + "' HEIGHT='" + (width / 2).ToString() + "' SRC='/DIOWave/downarrow.gif'></A></TD></TR>\n");

	for (int i = 0; i < sliderPanel.num_icons; i++) {
		Writer.Write("<TR><TD>");

		if (i == 0) {
			if (m_browser.type == "NN") {
				Writer.Write("<ILAYER ID='SLIDER-ANC' TOP='0' LEFT='0' WIDTH='0' HEIGHT='0' Z-INDEX='0' VISIBILITY='HIDDEN'></ILAYER>");
				Writer.Write("<LAYER ID='SLIDER-IND' TOP='" + (kInfoBarHeight + kMenuBarHeight + kStripWidth) + "' LEFT='" + (qsv.window_width - kStripWidth) + "' HEIGHT='0' WIDTH='0' Z-INDEX='1' VISIBILITY='HIDDEN' CLASS='SLIDER-IND'></LAYER>");
				Writer.Write("<LAYER ID='SLIDER-ACT' TOP='" + (kInfoBarHeight + kMenuBarHeight + kStripWidth) + "' LEFT='" + (qsv.window_width - kStripWidth) + "' HEIGHT='" + sliderPanel.panel_height + "' WIDTH='" + width + "' Z-INDEX='2' CLASS='SLIDER-ACT'></LAYER>");
			} else if (m_browser.type == "IE" && m_browser.os == "W") {
				Writer.Write("<DIV ID='SLIDER-IND' CLASS='SLIDER-IND'></DIV>");
				Writer.Write("<DIV ID='SLIDER-ACT' CLASS='SLIDER-ACT'>");
				if (m_browser.version >= 5.5)
					Writer.Write("<TABLE ALIGN='CENTER' HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' WIDTH='100%' HEIGHT='100%'><TR><TD></TD</TR></TABLE>");
				Writer.Write("</DIV>");
			}
		}

		UInt32 index = (UInt32)Math.Floor((Double)i * sliderPanel.icon_step);
		Writer.Write("<IMG NAME='ICN-" + index + "' HEIGHT='" + width + "' WIDTH='" + width + "' BORDER='0' HSPACE='0' VSPACE='0' SRC='/DIOWave/bbox.gif'>");

		Writer.Write("</TD></TR>\n");
	}

	Writer.Write("<TR><TD><IMG SRC='/DIOWave/grayscale.gif' HEIGHT='" + (height - sliderPanel.panel_height - width) + "' WIDTH='" + width + "' BORDER='0' HSPACE='0' VSPACE='0'></TD></TR>\n");

	Writer.Write("</TABLE>");
}

//==============================
//
//==============================
void printImageTable(LayoutEntry layout, UInt32 canvasHeight, UInt32 canvasWidth) {

	UInt32 tableWidth = layout.cols * layout.width + kImageSpacing * (layout.cols - 1);
	UInt32 tableHeight = layout.rows * layout.height + kImageSpacing * (layout.rows - 1);

	UInt32 hs = canvasWidth - tableWidth;
	UInt32 vs = canvasHeight - tableHeight;

	UInt32 hsl = (UInt32)Math.Floor((Double)hs / 2);
	UInt32 hsr = hs - hsl;

	UInt32 vst = (UInt32)Math.Floor((Double)vs / 2);
	UInt32 vsb = vs - vst;
	vst = 0;

	Writer.Write("<TABLE ALIGN='LEFT' HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' HEIGHT='" + canvasHeight + "' WIDTH='" + canvasWidth + "'>\n");

	if (vst > 0) {
		Writer.Write("<TR>\n");
		if (hsl > 0)
			Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + hsl + "'></TD>\n");
		for (UInt32 c = 0; c < layout.cols; c++) {
			if (c > 0)
				Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + kImageSpacing + "'></TD>\n");
			Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + layout.width + "'></TD>\n");
		}
		if (hsr > 0)
			Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + hsr + "'></TD>\n");
		Writer.Write("</TR>\n");
	}

	for (UInt32 r = 0; r < layout.rows; r++) {

		if (r > 0) {
			Writer.Write("<TR>\n");
			if (hsl > 0)
				Writer.Write("\t<TD HEIGHT='" + kImageSpacing + "' WIDTH='" + hsl + "'></TD>\n");
			for (UInt32 c = 0; c < layout.cols; c++) {
				if (c > 0)
					Writer.Write("\t<TD HEIGHT='" + kImageSpacing + "' WIDTH='" + kImageSpacing + "'></TD>\n");
				Writer.Write("\t<TD HEIGHT='" + kImageSpacing + "' WIDTH='" + layout.width + "'></TD>\n");
			}
			if (hsr > 0)
				Writer.Write("\t<TD HEIGHT='" + kImageSpacing + "' WIDTH='" + hsr + "'></TD>\n");
			Writer.Write("</TR>\n");
		}

		Writer.Write("<TR>\n");

		if (hsl > 0)
			Writer.Write("\t<TD HEIGHT='" + layout.height + "' WIDTH='" + hsl + "'></TD>\n");

		for (UInt32 c = 0; c < layout.cols; c++) {
			UInt32 index = layout.cols * r + c;

			if (c > 0)
				Writer.Write("\t<TD HEIGHT='" + layout.height + "' WIDTH='" + kImageSpacing + "'></TD>\n");

			Writer.Write("\t<TD HEIGHT='" + layout.height + "' WIDTH='" + layout.width + "'>");
			Writer.Write("<DIV ID='CTN-"+index+"' class='IMG-CONTAINER'>");
				Writer.Write("<DIV ID='ANN-" + index + "' CLASS='ANN'></DIV>");
				Writer.Write("<DIV ID='ACT-" + index + "' CLASS='ACT'>");
				if (m_browser.version >= 5.5) {
					Writer.Write("<TABLE HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' WIDTH='100%' HEIGHT='100%' STYLE='FONT-SIZE: "+layout.width+";'>");
					Writer.Write("<TR><TD ID='ACTTD-" + index + "'>&nbsp;</TD></TR>");
					Writer.Write("</TABLE>");
				}
				Writer.Write("</DIV>");


			Writer.Write("<IMG NAME='IMG-" + index + "' BORDER='0' HSPACE='0' VSPACE='0' WIDTH='" + layout.width + "' HEIGHT='" + layout.height + "' SRC='/DIOWave/bbox.gif'>");

			Writer.Write("</DIV>");
			Writer.Write("</TD>\n");
		}

		if (hsr > 0)
			Writer.Write("\t<TD HEIGHT='" + layout.height + "' WIDTH='" + hsr + "'></TD>\n");

		Writer.Write("</TR>\n");
	}

	if (vsb > 0) {
		Writer.Write("<TR>\n");
		if (hsl > 0)
			Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + hsl + "'></TD>\n");
		for (UInt32 c = 0; c < layout.cols; c++) {
			if (c > 0)
				Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + kImageSpacing + "'></TD>\n");
			Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + layout.width + "'></TD>\n");
		}
		if (hsr > 0)
			Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + hsr + "'></TD>\n");
		Writer.Write("</TR>\n");
	}

	Writer.Write("</TABLE>");
}


//==========================
//
//==========================
void print_pixel_info(SOPInstanceObject file) {
	int bitsStored = (int)get_data_US(file, null, 0x00280101, 0);
	int pixelRep = (int)get_data_US(file, null, 0x00280103, 0);

	Writer.Write("<NOBR>Bits Stored: <FONT COLOR='white'>" + bitsStored + " ");

	Writer.Write("[");
	if (pixelRep == 1)
		Writer.Write((-(1 << (bitsStored - 1))).ToString() + " .. " + ((1 << (bitsStored - 1)) - 1).ToString());
	else
		Writer.Write("0 .. " + ((1 << bitsStored) - 1).ToString());
	Writer.Write("]");

	Writer.Write("</FONT></NOBR>; \n");
}

//==========================
//
//==========================
void print_image_dimensions(SOPInstanceObject file) {

	ArrayList pixelSpacing = get_data_array(file, null, 0x00280030);
	if (pixelSpacing != null && pixelSpacing.Count == 2) {
		try {
			Double psx = Double.Parse((String)pixelSpacing[0], DS_ns);
			Double psy = Double.Parse((String)pixelSpacing[1], DS_ns);
			Writer.Write("<NOBR>Spacing: <FONT COLOR='white'>" + psx.ToString("F3") + " x " + psy.ToString("F3") + "</FONT></NOBR>; \n");
		} catch (FormatException fe) {
			// maybe it is empty DS elements
			// prevent to throw the exception to caller
		}
	}

	String sliceThickness = get_data_DS(file, null, 0x00180050, null);
	if (sliceThickness != null) {
		try {
			Double val = Double.Parse(sliceThickness, DS_ns);
			Writer.Write("<NOBR>Thickness: <FONT COLOR='white'>" + val.ToString("F2") + "</FONT></NOBR>; \n");
		} catch (FormatException fe) {
			// maybe it is empty DS elements
			// prevent to throw the exception to caller
		}
	}

}

//==============================
//
//==============================
void printSeriesDetail(DicomStudy dom, QueryStringValue qsv) {

	SOPInstanceObject file = (SOPInstanceObject)dom.fl[0];

	Writer.Write("<DIV CLASS='SeriesDetail' ID='SeriesDetail'>");

	print_pixel_info(file);
	print_image_dimensions(file);

	String modality = get_data_CS(file, dom, 0x00080060, null);
	if (modality == "MR") {
		print_desc_text(file, 0x00080008, "Image Type", 3);
		print_desc_text(file, 0x00181030, "Protocol Name", 0);
		print_desc_text(file, 0x0008103E, "Series Description", 0);
		print_desc_text(file, 0x00180010, "Contrast/Bolus Agent", 0);
		print_desc_text(file, 0x00185100, "Patient Position", 0);
		print_desc_text(file, 0x00180020, "Scanning Sequence", 0);
		print_desc_text(file, 0x00180021, "Sequence Variant", 0);
		print_desc_text(file, 0x00180022, "Scan Options", 0);
		print_desc_text(file, 0x00180023, "MR Acquisition Type", 0);
		print_desc_text(file, 0x00180024, "Sequence Name", 0);
		print_desc_text(file, 0x00180080, "Repetition Time", 0);
		print_desc_text(file, 0x00180081, "Echo Time", 0);
		print_desc_text(file, 0x00180082, "Inversion Time", 0);
		print_desc_text(file, 0x00180087, "Magnetic Field Strength", 0);
		print_desc_text(file, 0x00100030, "Patient's DOB", 0);
		print_desc_text(file, 0x00101010, "Patient's Age", 0);
		print_desc_text(file, 0x00100040, "Sex", 0);
		print_desc_text(file, 0x00080080, "Institution", 0);
	} else if (modality == "CT") {
		print_desc_text(file, 0x00080008, "Image Type", 3);
		print_desc_text(file, 0x00181030, "Protocol Name", 0);
		print_desc_text(file, 0x0008103E, "Series Description", 0);
		print_desc_text(file, 0x00180010, "Contrast/Bolus Agent", 0);
		print_desc_text(file, 0x00185100, "Patient Position", 0);
		print_desc_text(file, 0x00180060, "KVP", 0);
		print_desc_text(file, 0x00180090, "Data Collection Diameter", 0);
		print_desc_text(file, 0x00180090, "Reconstruction Diameter", 0);
		print_desc_text(file, 0x00180022, "Scan Options", 0);
		print_desc_text(file, 0x00181140, "Rotation Direction", 0);
		print_desc_text(file, 0x00181160, "Filter Type", 0);
		print_desc_text(file, 0x00181190, "Focal Spot", 0);
		print_desc_text(file, 0x00181210, "Convolution Kernel", 0);
		print_desc_text(file, 0x00100030, "Patient's DOB", 0);
		print_desc_text(file, 0x00101010, "Patient's Age", 0);
		print_desc_text(file, 0x00100040, "Sex", 0);
		print_desc_text(file, 0x00080080, "Institution", 0);
	}

	Writer.Write("</DIV>\n");
}

//==============================
//
//==============================
void printSeriesDetailButton() {
	Writer.Write("<A STYLE='color:white;' HREF='javascript:show_series_detail()'>Info</A>");
}
